﻿(function () {
    'use strict';

    angular.module('tubular.directives')
        /**
         * @ngdoc directive
         * @name tbGridPager
         * @restrict E
         *
         * @description
         * The `tbGridPager` directive generates a pager connected to the parent `tbGrid`.
         * 
         * @scope
         */
        .directive('tbGridPager', [
            '$timeout', function ($timeout) {
                return {
                    require: '^tbGrid',
                    template:
                        '<div class="tubular-pager">' +
                            '<uib-pagination ng-disabled="$component.isEmpty" direction-links="true" ' +
                            'boundary-links="true" total-items="$component.filteredRecordCount" ' +
                            'items-per-page="$component.pageSize" max-size="5" ng-model="$component.currentPage" ng-change="pagerPageChanged()">' +
                            '</uib-pagination>' +
                            '<div>',
                    restrict: 'E',
                    replace: true,
                    transclude: false,
                    scope: true,
                    terminal: false,
                    controller: [
                        '$scope', '$element', function ($scope, $element) {
                            $scope.$component = $scope.$parent.$parent;
                            $scope.tubularDirective = 'tubular-grid-pager';

                            $scope.$watch('$component.currentPage', function () {
                                if ($scope.$component.currentPage != $scope.$component.requestedPage) {
                                    $scope.$component.requestedPage = $scope.$component.currentPage;
                                }
                            });

                            $scope.pagerPageChanged = function () {
                                $scope.$component.requestedPage = $scope.$component.currentPage;
                                var allLinks = $element.find('li a');
                                $(allLinks).blur();
                            };
                        }
                    ],
                    compile: function compile() {
                        return {
                            post: function (scope, lElement, lAttrs) {
                                scope.firstButtonClass = lAttrs.firstButtonClass || 'fa fa-fast-backward';
                                scope.prevButtonClass = lAttrs.prevButtonClass || 'fa fa-backward';

                                scope.nextButtonClass = lAttrs.nextButtonClass || 'fa fa-forward';
                                scope.lastButtonClass = lAttrs.lastButtonClass || 'fa fa-fast-forward';

                                var timer = $timeout(function () {
                                    var allLinks = lElement.find('li a');

                                    $(allLinks[0]).html('<i class="' + scope.firstButtonClass + '"></i>');
                                    $(allLinks[1]).html('<i class="' + scope.prevButtonClass + '"></i>');

                                    $(allLinks[allLinks.length - 2]).html('<i class="' + scope.nextButtonClass + '"></i>');
                                    $(allLinks[allLinks.length - 1]).html('<i class="' + scope.lastButtonClass + '"></i>');
                                }, 0);

                                scope.$on('$destroy', function () { $timeout.cancel(timer); });
                            }
                        };
                    }
                };
            }
        ])
        /**
         * @ngdoc directive
         * @name tbGridPagerInfo
         * @restrict E
         *
         * @description
         * The `tbGridPagerInfo` directive shows how many records are shown in a page and total rows.
         * 
         * @scope
         */
        .directive('tbGridPagerInfo', [
            function () {
                return {
                    require: '^tbGrid',
                    template: '<div class="pager-info small" ng-hide="$component.isEmpty">' +
                        '{{\'UI_SHOWINGRECORDS\' | translate: currentInitial:currentTop:$component.filteredRecordCount}} ' +
                        '<span ng-show="filtered">' +
                        '{{\'UI_FILTEREDRECORDS\' | translate: $component.totalRecordCount}}</span>' +
                        '</div>',
                    restrict: 'E',
                    replace: true,
                    transclude: true,
                    scope: true,
                    controller: [
                        '$scope', function ($scope) {
                            $scope.$component = $scope.$parent.$parent;
                            $scope.fixCurrentTop = function () {
                                $scope.currentTop = $scope.$component.pageSize * $scope.$component.currentPage;
                                $scope.currentInitial = (($scope.$component.currentPage - 1) * $scope.$component.pageSize) + 1;

                                if ($scope.currentTop > $scope.$component.filteredRecordCount) {
                                    $scope.currentTop = $scope.$component.filteredRecordCount;
                                }

                                if ($scope.currentTop < 0) {
                                    $scope.currentTop = 0;
                                }

                                if ($scope.currentInitial < 0 || $scope.$component.totalRecordCount === 0) {
                                    $scope.currentInitial = 0;
                                }
                            };

                            $scope.$component.$watch('filteredRecordCount', function () {
                                $scope.filtered = $scope.$component.totalRecordCount != $scope.$component.filteredRecordCount;
                                $scope.fixCurrentTop();
                            });

                            $scope.$component.$watch('currentPage', function () {
                                $scope.fixCurrentTop();
                            });

                            $scope.$component.$watch('pageSize', function () {
                                $scope.fixCurrentTop();
                            });

                            $scope.fixCurrentTop();
                        }
                    ]
                };
            }
        ]);
})();